@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */
+    along with this program; If not, see <https://www.gnu.org/licenses/>. */
 
 
 #include "sed.h"
@@ -25,9 +24,11 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include "binary-io.h"
 #include "getopt.h"
 #include "progname.h"
 #include "version.h"
+#include "xalloc.h"
 
 #include "version-etc.h"
 
@@ -35,7 +36,9 @@
    _("Jay Fenlason"), \
    _("Tom Lord"), \
    _("Ken Pizzini"), \
-   _("Paolo Bonzini")
+   _("Paolo Bonzini"), \
+   _("Jim Meyering"), \
+   _("Assaf Gordon")
 
 int extended_regexp_flags = 0;
 
@@ -57,6 +60,9 @@ bool follow_symlinks = false;
 /* If set, opearate in 'sandbox' mode */
 bool sandbox = false;
 
+/* if set, print debugging information */
+bool debug = false;
+
 /* How do we edit files in-place? (we don't if NULL) */
 char *in_place_extension = NULL;
 
@@ -64,6 +70,11 @@ char *in_place_extension = NULL;
 char const *read_mode = "r";
 char const *write_mode = "w";
 
+#if O_BINARY
+/* Additional flag for binary mode on platforms with O_BINARY/O_TEXT.  */
+bool binary_mode = false;
+#endif
+
 /* Do we need to be pedantically POSIX compliant? */
 enum posixicity_types posixicity;
 
@@ -86,6 +97,7 @@ struct localeinfo localeinfo;
 static void
 cleanup (void)
 {
+  IF_LINT (free (in_place_extension));
   if (G_file_to_unlink)
     unlink (G_file_to_unlink);
 }
@@ -106,77 +118,67 @@ cancel_cleanup (void)
 
 static void usage (int);
 static void
-contact(int errmsg)
+contact (int errmsg)
 {
   FILE *out = errmsg ? stderr : stdout;
-#ifndef REG_PERL
-  fprintf(out, _("GNU sed home page: <https://www.gnu.org/software/sed/>.\n\
+  fprintf (out, _("GNU sed home page: <https://www.gnu.org/software/sed/>.\n\
 General help using GNU software: <https://www.gnu.org/gethelp/>.\n"));
-#endif
 
   /* Only print the bug report address for `sed --help', otherwise we'll
      get reports for other people's bugs.  */
   if (!errmsg)
-    fprintf(out, _("E-mail bug reports to: <%s>.\n"), PACKAGE_BUGREPORT);
+    fprintf (out, _("E-mail bug reports to: <%s>.\n"), PACKAGE_BUGREPORT);
 }
 
 _Noreturn static void
-usage(int status)
+usage (int status)
 {
   FILE *out = status ? stderr : stdout;
 
-#ifdef REG_PERL
-#define PERL_HELP _("  -R, --regexp-perl" \
-                    "\n                 use Perl 5's regular expressions" \
-                    " syntax in the script.\n")
-#endif
-
-  fprintf(out, _("\
+  fprintf (out, _("\
 Usage: %s [OPTION]... {script-only-if-no-other-script} [input-file]...\n\
-\n"), myname);
+\n"), program_name);
 
-  fprintf(out, _("  -n, --quiet, --silent\n\
+  fprintf (out, _("  -n, --quiet, --silent\n\
                  suppress automatic printing of pattern space\n"));
-  fprintf(out, _("  -e script, --expression=script\n\
+  fprintf (out, _("      --debug\n\
+                 annotate program execution\n"));
+  fprintf (out, _("  -e script, --expression=script\n\
                  add the script to the commands to be executed\n"));
-  fprintf(out, _("  -f script-file, --file=script-file\n\
+  fprintf (out, _("  -f script-file, --file=script-file\n\
                  add the contents of script-file to the commands" \
                  " to be executed\n"));
 #ifdef ENABLE_FOLLOW_SYMLINKS
-  fprintf(out, _("  --follow-symlinks\n\
+  fprintf (out, _("  --follow-symlinks\n\
                  follow symlinks when processing in place\n"));
 #endif
-  fprintf(out, _("  -i[SUFFIX], --in-place[=SUFFIX]\n\
+  fprintf (out, _("  -i[SUFFIX], --in-place[=SUFFIX]\n\
                  edit files in place (makes backup if SUFFIX supplied)\n"));
-#if defined WIN32 || defined _WIN32 || defined __CYGWIN__ \
-  || defined MSDOS || defined __EMX__
-  fprintf(out, _("  -b, --binary\n\
+#if O_BINARY
+  fprintf (out, _("  -b, --binary\n\
                  open files in binary mode (CR+LFs are not" \
                  " processed specially)\n"));
 #endif
-  fprintf(out, _("  -l N, --line-length=N\n\
+  fprintf (out, _("  -l N, --line-length=N\n\
                  specify the desired line-wrap length for the `l' command\n"));
-  fprintf(out, _("  --posix\n\
+  fprintf (out, _("  --posix\n\
                  disable all GNU extensions.\n"));
-  fprintf(out, _("  -E, -r, --regexp-extended\n\
+  fprintf (out, _("  -E, -r, --regexp-extended\n\
                  use extended regular expressions in the script\n\
                  (for portability use POSIX -E).\n"));
-#ifdef REG_PERL
-  fprintf(out, PERL_HELP);
-#endif
-  fprintf(out, _("  -s, --separate\n\
+  fprintf (out, _("  -s, --separate\n\
                  consider files as separate rather than as a single,\n\
                  continuous long stream.\n"));
-  fprintf(out, _("      --sandbox\n\
+  fprintf (out, _("      --sandbox\n\
                  operate in sandbox mode (disable e/r/w commands).\n"));
-  fprintf(out, _("  -u, --unbuffered\n\
+  fprintf (out, _("  -u, --unbuffered\n\
                  load minimal amounts of data from the input files and flush\n\
                  the output buffers more often\n"));
-  fprintf(out, _("  -z, --null-data\n\
+  fprintf (out, _("  -z, --null-data\n\
                  separate lines by NUL characters\n"));
-  fprintf(out, _("      --help     display this help and exit\n"));
-  fprintf(out, _("      --version  output version information and exit\n"));
-  fprintf(out, _("\n\
+  fprintf (out, _("      --help     display this help and exit\n"));
+  fprintf (out, _("      --version  output version information and exit\n"));
+  fprintf (out, _("\n\
 If no -e, --expression, -f, or --file option is given, then the first\n\
 non-option argument is taken as the sed script to interpret.  All\n\
 remaining arguments are names of input files; if no input files are\n\
@@ -191,20 +193,16 @@ specified, then the standard input is read.\n\
 int
 main (int argc, char **argv)
 {
-#ifdef REG_PERL
-#define SHORTOPTS "bsnrzRuEe:f:l:i::V:"
-#else
 #define SHORTOPTS "bsnrzuEe:f:l:i::V:"
-#endif
 
-  enum { SANDBOX_OPTION = CHAR_MAX+1 };
+  enum { SANDBOX_OPTION = CHAR_MAX+1,
+         DEBUG_OPTION
+    };
 
   static const struct option longopts[] = {
     {"binary", 0, NULL, 'b'},
     {"regexp-extended", 0, NULL, 'r'},
-#ifdef REG_PERL
-    {"regexp-perl", 0, NULL, 'R'},
-#endif
+    {"debug", 0, NULL, DEBUG_OPTION},
     {"expression", 1, NULL, 'e'},
     {"file", 1, NULL, 'f'},
     {"in-place", 2, NULL, 'i'},
@@ -227,15 +225,14 @@ main (int argc, char **argv)
 
   int opt;
   int return_code;
-  const char *cols = getenv("COLS");
+  const char *cols = getenv ("COLS");
 
-  program_name = argv[0];
+  set_program_name (argv[0]);
   initialize_main (&argc, &argv);
 #if HAVE_SETLOCALE
   /* Set locale according to user's wishes.  */
   setlocale (LC_ALL, "");
 #endif
-  set_program_name (argv[0]);
   initialize_mbcs ();
   init_localeinfo (&localeinfo);
 
@@ -250,7 +247,7 @@ main (int argc, char **argv)
   textdomain (PACKAGE);
 #endif
 
-  if (getenv("POSIXLY_CORRECT") != NULL)
+  if (getenv ("POSIXLY_CORRECT") != NULL)
     posixicity = POSIXLY_CORRECT;
   else
     posixicity = POSIXLY_EXTENDED;
@@ -261,13 +258,12 @@ main (int argc, char **argv)
    */
   if (cols)
     {
-      countT t = atoi(cols);
+      countT t = atoi (cols);
       if (t > 1)
         lcmd_out_line_len = t-1;
     }
 
-  myname = *argv;
-  while ((opt = getopt_long(argc, argv, SHORTOPTS, longopts, NULL)) != EOF)
+  while ((opt = getopt_long (argc, argv, SHORTOPTS, longopts, NULL)) != EOF)
     {
       switch (opt)
         {
@@ -275,10 +271,10 @@ main (int argc, char **argv)
           no_default_output = true;
           break;
         case 'e':
-          the_program = compile_string(the_program, optarg, strlen(optarg));
+          the_program = compile_string (the_program, optarg, strlen (optarg));
           break;
         case 'f':
-          the_program = compile_file(the_program, optarg);
+          the_program = compile_file (the_program, optarg);
           break;
 
         case 'z':
@@ -291,16 +287,17 @@ main (int argc, char **argv)
 
         case 'i':
           separate_files = true;
+          IF_LINT (free (in_place_extension));
           if (optarg == NULL)
             /* use no backups */
-            in_place_extension = ck_strdup ("*");
+            in_place_extension = xstrdup ("*");
 
-          else if (strchr(optarg, '*') != NULL)
-            in_place_extension = ck_strdup(optarg);
+          else if (strchr (optarg, '*') != NULL)
+            in_place_extension = xstrdup (optarg);
 
           else
             {
-              in_place_extension = MALLOC (strlen(optarg) + 2, char);
+              in_place_extension = XCALLOC (strlen (optarg) + 2, char);
               in_place_extension[0] = '*';
               strcpy (in_place_extension + 1, optarg);
             }
@@ -308,7 +305,7 @@ main (int argc, char **argv)
           break;
 
         case 'l':
-          lcmd_out_line_len = atoi(optarg);
+          lcmd_out_line_len = atoi (optarg);
           break;
 
         case 'p':
@@ -318,25 +315,16 @@ main (int argc, char **argv)
         case 'b':
           read_mode = "rb";
           write_mode = "wb";
+#if O_BINARY
+          binary_mode = true;
+#endif
           break;
 
         case 'E':
         case 'r':
-#ifdef REG_PERL
-          if (extended_regexp_flags && (extended_regexp_flags!=REG_EXTENDED))
-            usage(EXIT_BAD_USAGE);
-#endif
           extended_regexp_flags = REG_EXTENDED;
           break;
 
-#ifdef REG_PERL
-        case 'R':
-          if (extended_regexp_flags && (extended_regexp_flags!=REG_PERL)))
-            usage(EXIT_BAD_USAGE);
-          extended_regexp_flags = REG_PERL;
-          break;
-#endif
-
         case 's':
           separate_files = true;
           break;
@@ -345,20 +333,24 @@ main (int argc, char **argv)
           sandbox = true;
           break;
 
+        case DEBUG_OPTION:
+          debug = true;
+          break;
+
         case 'u':
           unbuffered = true;
           break;
 
         case 'v':
-          version_etc(stdout, program_name, PACKAGE_NAME, Version,
+          version_etc (stdout, program_name, PACKAGE_NAME, Version,
                       AUTHORS, (char *) NULL);
-          contact(false);
+          contact (false);
           ck_fclose (NULL);
           exit (EXIT_SUCCESS);
         case 'h':
-          usage(EXIT_SUCCESS);
+          usage (EXIT_SUCCESS);
         default:
-          usage(EXIT_BAD_USAGE);
+          usage (EXIT_BAD_USAGE);
         }
     }
 
@@ -367,17 +359,30 @@ main (int argc, char **argv)
       if (optind < argc)
         {
           char *arg = argv[optind++];
-          the_program = compile_string(the_program, arg, strlen(arg));
+          the_program = compile_string (the_program, arg, strlen (arg));
         }
       else
-        usage(EXIT_BAD_USAGE);
+        usage (EXIT_BAD_USAGE);
     }
-  check_final_program(the_program);
+  check_final_program (the_program);
 
-  return_code = process_files(the_program, argv+optind);
+#if O_BINARY
+  if (binary_mode)
+    {
+       if (set_binary_mode ( fileno (stdin), O_BINARY) == -1)
+         panic (_("failed to set binary mode on STDIN"));
+       if (set_binary_mode ( fileno (stdout), O_BINARY) == -1)
+         panic (_("failed to set binary mode on STDOUT"));
+    }
+#endif
 
-  finish_program();
-  ck_fclose(NULL);
+  if (debug)
+    debug_print_program (the_program);
+
+  return_code = process_files (the_program, argv+optind);
+
+  finish_program (the_program);
+  ck_fclose (NULL);
 
   return return_code;
 }
