@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 # Test misc.
 
-# Copyright (C) 2017 Free Software Foundation, Inc.
+# Copyright (C) 2017-2018 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use File::stat;
@@ -1190,6 +1190,13 @@ s,.*[^\/],,
            . "baaaacx\n"}
      ],
 
+
+     # Four backslashes (2 pairs of "\\") to pass through two interpolations:
+     # once in Perl, then the shell command line argument.
+     # sed will see one backslash character in the s/// command.
+     ['bug30794_1', "s/z/\\\\x5cA/",  {IN=>'z'}, {OUT => "\\A"}],
+     ['bug30794_2', "s/z/\\\\x5c/",   {IN=>'z'}, {OUT => "\\"}],
+     ['bug30794_3', "s/z/\\\\x5c1/",  {IN=>'z'}, {OUT => "\\1"}],
     );
 
 my $save_temps = $ENV{SAVE_TEMPS};
